package com.seeyon.ctp.common.util;

import com.seeyon.ctp.common.log.CtpLogFactory;
import org.apache.commons.logging.Log;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class XmlUtil {
    private static final Log log = CtpLogFactory.getLog(XmlUtil.class);

        /**
         * 将xml格式响应报文解析为Map格式
         *
         * @param responseXmlTemp
         * @return
         * @throws DocumentException
         */
        public static Map<String, Object> xml2map(String responseXmlTemp) throws DocumentException {
            Document doc = DocumentHelper.parseText(responseXmlTemp);
            Element rootElement = doc.getRootElement();
            Map<String, Object> mapXml = new HashMap<>();
            elementMap(mapXml, rootElement);
            return mapXml;
        }

        /**
         * 使用递归调用将多层级xml转为map
         *
         * @param map
         * @param rootElement
         */
        private static void elementMap(Map<String, Object> map, Element rootElement) {
            Element privateKey = (Element)rootElement.selectSingleNode("./privateKey");
            map.put(privateKey.getName(),privateKey.getText());
            Element syncContent = (Element)rootElement.selectSingleNode("./dataContent/syncContent");
            map.put("dataType",syncContent.attributeValue("dataType"));
            map.put("operType",syncContent.attributeValue("operType"));

            Map<String, Object> syncContentMap = new HashMap<>();
            Element syncUnicode = (Element)syncContent.selectSingleNode("./syncUnicode");
            syncContentMap.put(syncUnicode.getName(),syncUnicode.getText());
            Element oldContent = (Element)syncContent.selectSingleNode("./oldContent");
            syncContentMap.put("oldContent",element2Map(oldContent));
            Element changeContent = (Element)syncContent.selectSingleNode("./changeContent");
            syncContentMap.put("changeContent",element2Map(changeContent));
            Element newContent  = (Element)syncContent.selectSingleNode("./newContent");
            syncContentMap.put("newContent",element2Map(newContent));

            map.put("syncContent",syncContentMap);
            // 获得当前节点的子节点
//            List<Element> childElements = rootElement.elements();
//            log.info("element name :" + rootElement.getName() + " -- childElements size：" + childElements.size());
//            if (childElements.size() > 0) {
//                Map<String, Object> tempMap = new HashMap<>();
//                for (Element e : childElements) {
//                    element2Map(tempMap, e);
//                    map.put(rootElement.getName(), tempMap);
//                }
//            } else {
//                log.info("当前无子节点 name :" + rootElement.getName() + ":" + rootElement.getText());
//                map.put(rootElement.getName(), rootElement.getText());
//            }
        }

    private static Map<String,Object> element2Map( Element element) {
        Map<String, Object> map = new HashMap<>();
        Element baseInfo = (Element) element.selectSingleNode("./baseInfo");
        if (baseInfo != null){
            List childElements = baseInfo.elements();
            if (childElements.size() > 0) {
                Map<String, Object> tempMap = new HashMap<>();
                Iterator var33 = childElements.iterator();
                while(var33.hasNext()) {
                    Element element1 = (Element)var33.next();
                    tempMap.put(element1.getName(),element1.getText());
                }
                map.put("baseInfo",tempMap);
            }
        }
        Element parentInfo = (Element) element.selectSingleNode("./parentInfo");
        if (parentInfo != null){
            Map<String, Object> tempMap = new HashMap<>();
            Element parentCode = (Element) parentInfo.selectSingleNode("./parentCode");
            if (parentCode != null) {
                tempMap.put(parentCode.getName(), parentCode.getText());
//            List childElements1 = parentInfo.elements();
//            if (childElements1.size() > 0) {
//                Iterator var44 = childElements1.iterator();
//                while(var44.hasNext()) {
//                    Element element1 = (Element)var44.next();
//                    tempMap.put(element1.getName(),element1.getText());
//                }
//
//                Element parentCode = (Element) childElements1.get(0).selectSingleNode("./parentInfo");
//                tempMap.put("isCorp",syncContent.attributeValue("isCorp"));
//                map.put("parentInfo",tempMap);
//            }
                tempMap.put("isCorp", parentCode.attributeValue("isCorp"));
                Element corpName = (Element) parentInfo.selectSingleNode("./corpName");
                if (corpName != null)
                    tempMap.put(corpName.getName(), corpName.getText());
                Element showNum = (Element) parentInfo.selectSingleNode("./showNum");
                if (showNum != null)
                    tempMap.put(showNum.getName(), showNum.getText());
            }
            map.put("parentInfo",tempMap);
        }
        return map;
    }

    public static String recursionMapToXml(Map<String ,Object> map){
        Document document = DocumentHelper.createDocument();
        Element root = document.addElement("returnData");
        for (String key: map.keySet()) {
            String value = map.get(key).toString();
            if (value == null) {
                value = "";
            }
            value = value.trim();
            Element filed = root.addElement(key);
            filed.setText(value);
        }
        return document.asXML();
    }
}
